@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 import os
 import shutil
-import subprocess  # nosec: disable=B603
+import subprocess  # nosec B404 B603
 import sys
 import tempfile
 import typing
